function [x, v] = sparseLS_GMC_ver2(y, A, AH, rho, lam, gamma)

% [x, v] = sparseLS_GMC_ver2(y, A, AH, rho, lam, gamma)
%
% Sparse Least-Squares with generalized MC (GMC) penalty
%
% Use FBS minimization - two L1-norm problems per iteration.
%
% INPUT
%   y 	    data
%   A, AH   operators for A and A^H
%   rho     rho >= maximum eigenvalue of A^H A
%   lam     regularization parameter, lam > 0
%   gamma   0 <= gamma < 1
%
% OUTPUT
%   x, v
%
% Ivan Selesnick
% March 2022
% 
% Reference:
% I. Selesnick, 'Sparse Regularization via Convex Analysis'
% IEEE Transactions on Signal Processing, 2017.
% 
% and other notes 


MAX_ITER = 10000;
TOL_STOP = 1e-4;

AHy = AH(y);

% initialization
x = zeros(size(AHy));
v = zeros(size(AHy));

iter = 0;
old_x = x;
delta_x = inf;

while (delta_x > TOL_STOP) && (iter < MAX_ITER)    
    iter = iter + 1;
    
    v = sparseLS_L1(A(x), A, AH, rho, lam/gamma, v);
    z = gamma * A(x - v);
    x = sparseLS_L1(y + z, A, AH, rho, lam, x);

    delta_x = max(abs( x(:) - old_x(:) )) / max(abs(old_x(:)));
    old_x = x;    
end

